#!/usr/local/bin/perl
#******************************************************************************
#
#       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
#       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
#       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
#       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
#       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
#       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
#       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
#       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
#       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
#       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
#       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
#       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#       FOR A PARTICULAR PURPOSE.
#
#       (c) Copyright 2007 Xilinx Inc.
#       All rights reserved.
#
#
#******************************************************************************
$mpmc_pcore = "MPMCv4";
use File::Copy;

$usage = "This program converts a Virtex5 DDR2 mpmc_v3_00_a or b .ucf file to a $mpmc_pcore and later .ucf file

example usage: 
 revup_v5_ucf.pl <ucf_input_file> <part_number> <dq_pin_name> <dqsn_pin_name> <ucf_output_file> <mhs_output_file>

where:
 <ucf_input_file> is the name of the .ucf file from mpmc_v3_00_a/b
 <part_number> is the FPGA part you are targeting.  For example xc5vlx50ff676.
 <dq_pin_name> is the base name of your DQ pins in your UCF file.  For example DDR2_DQ
 <dqsn_pin_name> is the base name of your DQS_N pins in your UCF file.  For example DDR2_DQS_N
 <ucf_output_file> is the desired name of the output .ucf file. 
 <mhs_output_file> is the desired name of the output MHS snippet that will need to be copied into the MPMC instantiation inside your MHS file. 
\n";

if ((!$ARGV[0]) || (!$ARGV[1]) || (!$ARGV[2]) || (!$ARGV[3]) || (!$ARGV[4]) || (!$ARGV[5]))
{
    die $usage;
}

$input_file = $ARGV[0];
$part_number = $ARGV[1];
$dq_pin_name = $ARGV[2];
$dqsn_pin_name = $ARGV[3];
$output_file = $ARGV[4];
$output_file_tmp1 = "$output_file.tmp1";
$output_file_tmp2 = "$output_file.tmp2";
$output_file_tmp3 = "$output_file.tmp3";
$mhs_output_file = $ARGV[5];
$target_clk_freq = 266;
$clk_period_constraint = "TS_SYS_CLK";

open (IN, $input_file) ||
    die "ERROR: Couldn't open input data file $input_file\n\n$usage";

#Find where the script lives, to run other scripts later with
($script_dir) = $0 =~ /^(.*[\/|\\])/;

die "ERROR mig_20_ucf_rev.pl not found in $script_dir: $!" unless (-e "${script_dir}mig20_ucf_rev.pl");
die "ERROR convert_ucf.pl not found in $script_dir: $!" unless (-e "${script_dir}convert_ucf.pl");

$myfile_list[0] = $output_file;
$myfile_list[1] = $output_file_tmp1;
$myfile_list[2] = $output_file_tmp2;
$myfile_list[3] = $output_file_tmp3;
$myfile_list[4] = $mhs_output_file;
$num=0;
foreach $i (@myfile_list) {
    $answer = "";
    while ((-e $i) && ($answer ne "y"))
    {
	while (!(($answer eq "y") || ($answer eq "n")))
	{
	    print "$i already exists, do you want to overwrite the existing file? (y/n)";
	    $answer = <STDIN>;
	    $answer =~ s/\s*$//g;
	    $answer = lc($answer);
	}
	if ($answer eq "n")
	{
	    print "please enter a new name for the output file: ";
	    if ($num == 0) {
		$output_file = <STDIN>;
		$output_file =~ s/\s*$//g;
	    } elsif ($num == 1) {
		$output_file_tmp1 = <STDIN>;
		$output_file_tmp1 =~ s/\s*$//g;
	    } elsif ($num == 2) {
		$output_file_tmp2 = <STDIN>;
		$output_file_tmp2 =~ s/\s*$//g;
	    } elsif ($num == 3) {
		$output_file_tmp3 = <STDIN>;
		$output_file_tmp3 =~ s/\s*$//g;
	    } elsif ($num == 4) {
		$mhs_output_file = <STDIN>;
		$mhs_output_file =~ s/\s*$//g;
	    }
	}
    }
    $num++;
}

$conversions{"$dq_pin_name<0>"} = "ddr2_dq[0]";
$conversions{"$dq_pin_name<1>"} = "ddr2_dq[1]";
$conversions{"$dq_pin_name<2>"} = "ddr2_dq[2]";
$conversions{"$dq_pin_name<3>"} = "ddr2_dq[3]";
$conversions{"$dq_pin_name<4>"} = "ddr2_dq[4]";
$conversions{"$dq_pin_name<5>"} = "ddr2_dq[5]";
$conversions{"$dq_pin_name<6>"} = "ddr2_dq[6]";
$conversions{"$dq_pin_name<7>"} = "ddr2_dq[7]";
$conversions{"$dq_pin_name<8>"} = "ddr2_dq[8]";
$conversions{"$dq_pin_name<9>"} = "ddr2_dq[9]";
$conversions{"$dq_pin_name<10>"} = "ddr2_dq[10]";
$conversions{"$dq_pin_name<11>"} = "ddr2_dq[11]";
$conversions{"$dq_pin_name<12>"} = "ddr2_dq[12]";
$conversions{"$dq_pin_name<13>"} = "ddr2_dq[13]";
$conversions{"$dq_pin_name<14>"} = "ddr2_dq[14]";
$conversions{"$dq_pin_name<15>"} = "ddr2_dq[15]";
$conversions{"$dq_pin_name<16>"} = "ddr2_dq[16]";
$conversions{"$dq_pin_name<17>"} = "ddr2_dq[17]";
$conversions{"$dq_pin_name<18>"} = "ddr2_dq[18]";
$conversions{"$dq_pin_name<19>"} = "ddr2_dq[19]";
$conversions{"$dq_pin_name<20>"} = "ddr2_dq[20]";
$conversions{"$dq_pin_name<21>"} = "ddr2_dq[21]";
$conversions{"$dq_pin_name<22>"} = "ddr2_dq[22]";
$conversions{"$dq_pin_name<23>"} = "ddr2_dq[23]";
$conversions{"$dq_pin_name<24>"} = "ddr2_dq[24]";
$conversions{"$dq_pin_name<25>"} = "ddr2_dq[25]";
$conversions{"$dq_pin_name<26>"} = "ddr2_dq[26]";
$conversions{"$dq_pin_name<27>"} = "ddr2_dq[27]";
$conversions{"$dq_pin_name<28>"} = "ddr2_dq[28]";
$conversions{"$dq_pin_name<29>"} = "ddr2_dq[29]";
$conversions{"$dq_pin_name<30>"} = "ddr2_dq[30]";
$conversions{"$dq_pin_name<31>"} = "ddr2_dq[31]";
$conversions{"$dq_pin_name<32>"} = "ddr2_dq[32]";
$conversions{"$dq_pin_name<33>"} = "ddr2_dq[33]";
$conversions{"$dq_pin_name<34>"} = "ddr2_dq[34]";
$conversions{"$dq_pin_name<35>"} = "ddr2_dq[35]";
$conversions{"$dq_pin_name<36>"} = "ddr2_dq[36]";
$conversions{"$dq_pin_name<37>"} = "ddr2_dq[37]";
$conversions{"$dq_pin_name<38>"} = "ddr2_dq[38]";
$conversions{"$dq_pin_name<39>"} = "ddr2_dq[39]";
$conversions{"$dq_pin_name<40>"} = "ddr2_dq[40]";
$conversions{"$dq_pin_name<41>"} = "ddr2_dq[41]";
$conversions{"$dq_pin_name<42>"} = "ddr2_dq[42]";
$conversions{"$dq_pin_name<43>"} = "ddr2_dq[43]";
$conversions{"$dq_pin_name<44>"} = "ddr2_dq[44]";
$conversions{"$dq_pin_name<45>"} = "ddr2_dq[45]";
$conversions{"$dq_pin_name<46>"} = "ddr2_dq[46]";
$conversions{"$dq_pin_name<47>"} = "ddr2_dq[47]";
$conversions{"$dq_pin_name<48>"} = "ddr2_dq[48]";
$conversions{"$dq_pin_name<49>"} = "ddr2_dq[49]";
$conversions{"$dq_pin_name<50>"} = "ddr2_dq[50]";
$conversions{"$dq_pin_name<51>"} = "ddr2_dq[51]";
$conversions{"$dq_pin_name<52>"} = "ddr2_dq[52]";
$conversions{"$dq_pin_name<53>"} = "ddr2_dq[53]";
$conversions{"$dq_pin_name<54>"} = "ddr2_dq[54]";
$conversions{"$dq_pin_name<55>"} = "ddr2_dq[55]";
$conversions{"$dq_pin_name<56>"} = "ddr2_dq[56]";
$conversions{"$dq_pin_name<57>"} = "ddr2_dq[57]";
$conversions{"$dq_pin_name<58>"} = "ddr2_dq[58]";
$conversions{"$dq_pin_name<59>"} = "ddr2_dq[59]";
$conversions{"$dq_pin_name<60>"} = "ddr2_dq[60]";
$conversions{"$dq_pin_name<61>"} = "ddr2_dq[61]";
$conversions{"$dq_pin_name<62>"} = "ddr2_dq[62]";
$conversions{"$dq_pin_name<63>"} = "ddr2_dq[63]";
$conversions{"$dq_pin_name<64>"} = "ddr2_dq[64]";
$conversions{"$dq_pin_name<65>"} = "ddr2_dq[65]";
$conversions{"$dq_pin_name<66>"} = "ddr2_dq[66]";
$conversions{"$dq_pin_name<67>"} = "ddr2_dq[67]";
$conversions{"$dq_pin_name<68>"} = "ddr2_dq[68]";
$conversions{"$dq_pin_name<69>"} = "ddr2_dq[69]";
$conversions{"$dq_pin_name<70>"} = "ddr2_dq[70]";
$conversions{"$dq_pin_name<71>"} = "ddr2_dq[71]";
$conversions{"$dq_pin_name<72>"} = "ddr2_dq[72]";
$conversions{"$dqsn_pin_name<0>"} = "ddr2_dqs_n[0]";
$conversions{"$dqsn_pin_name<1>"} = "ddr2_dqs_n[1]";
$conversions{"$dqsn_pin_name<2>"} = "ddr2_dqs_n[2]";
$conversions{"$dqsn_pin_name<3>"} = "ddr2_dqs_n[3]";
$conversions{"$dqsn_pin_name<4>"} = "ddr2_dqs_n[4]";
$conversions{"$dqsn_pin_name<5>"} = "ddr2_dqs_n[5]";
$conversions{"$dqsn_pin_name<6>"} = "ddr2_dqs_n[6]";
$conversions{"$dqsn_pin_name<7>"} = "ddr2_dqs_n[7]";
$conversions{"$dqsn_pin_name<8>"} = "ddr2_dqs_n[8]";

open (OUT, ">$output_file_tmp1") ||
    die "ERROR: Couldn't open output file\n";

while ($line = <IN>)                   #go through the data file
{
    foreach $key (reverse sort (keys %conversions))
    {
	$mod_key = $key;
	$mod_key =~ s/([\*|\[|\]|\(|\)|\{|\}|\?])/\\$1/g;
	if ($line =~ /$mod_key/i)
	{
	    $line =~ s/$mod_key/$conversions{$key}/;
	}
    }
    print OUT $line;
}

close (IN);
close (OUT);

#Run the MIG 2.0 V5 upgrade script
system("xilperl ${script_dir}mig20_ucf_rev.pl $output_file_tmp1 $part_number $target_clk_freq $output_file_tmp2") && die "ERROR: MIG revup failed";

#Run the MPMC MIG2.0-> MPMCv4 convert script
system("xilperl ${script_dir}convert_ucf.pl $output_file_tmp2.ucf $output_file_tmp3") && die "MPMC MIG conversion failed";

#Concatenate the new constraints to the ucf
copy ($input_file, $output_file) or die $!;
open (IN, $output_file_tmp3) or die $!;
open (OUT,">>$output_file") or die $!;
while (<IN>) {
    print OUT;
}
close(IN);
close(OUT);

#Grab the lines with 'parameter DQ' in it and write the lines to a MHS snippet
my $mygrep = 'parameter DQ';
open (IN, "${output_file_tmp2}_hdl_params.txt") or die $!;
open (OUT,">$mhs_output_file.tmp") or die $!;
while (<IN>) {
      print OUT if (/$mygrep/);  
}
close(IN);
close(OUT);

$conversions2{"parameter"} = "PARAMETER";
$conversions2{","} = " #";
$conversions2{"DQS_IO_COL = 18\'b"} = "C_MEM_DQS_IO_COL = 0b";
$conversions2{"DQS_IO_COL = 16\'b"} = "C_MEM_DQS_IO_COL = 0b00";
$conversions2{"DQS_IO_COL = 8\'b"} = "C_MEM_DQS_IO_COL = 0b0000000000";
$conversions2{"DQS_IO_COL = 4\'b"} = "C_MEM_DQS_IO_COL = 0b00000000000000";
$conversions2{"DQS_IO_COL = 2\'b"} = "C_MEM_DQS_IO_COL = 0b0000000000000000";
$conversions2{"DQS_IO_COL = 1\'b"} = "C_MEM_DQS_IO_COL = 0b00000000000000000";
$conversions2{"DQ_IO_MS = 72\'b"} = "C_MEM_DQ_IO_MS = 0b";
$conversions2{"DQ_IO_MS = 64\'b"} = "C_MEM_DQ_IO_MS = 0b00000000";
$conversions2{"DQ_IO_MS = 32\'b"} = "C_MEM_DQ_IO_MS = 0b0000000000000000000000000000000000000000";
$conversions2{"DQ_IO_MS = 16\'b"} = "C_MEM_DQ_IO_MS = 0b00000000000000000000000000000000000000000000000000000000";
$conversions2{"DQ_IO_MS = 8\'b"} = "C_MEM_DQ_IO_MS = 0b0000000000000000000000000000000000000000000000000000000000000000";
$mhs_output_file_tmp = "$mhs_output_file.tmp";
open (IN, $mhs_output_file_tmp) ||
    die "ERROR: Couldn't open input data file $mhs_output_file_tmp\n";
open (OUT, ">$mhs_output_file") ||
    die "ERROR: Couldn't open output file $mhs_output_file\n";
print OUT "# Virtex-5 DDR2 revup MHS additions\n";
print OUT "# Add the following ports and attributes to the MPMC instance\n";
print OUT "   PORT MPMC_Clk0_DIV2 = <insert clock net name here> # MPMC_Clk0, divided by 2\n";
while (<IN>)                   #go through the data file
{
    foreach $key (reverse sort (keys %conversions2))
    {
	$mod_key = $key;
	$mod_key =~ s/([\*|\[|\]|\(|\)|\{|\}|\?])/\\$1/g;
	if (/$mod_key/i)
	{
	    s/$mod_key/$conversions2{$key}/;
	}
    }
    print OUT;
}
close (IN);
close (OUT);

print <<"END_OF_SUMMARY";

//------------------------------------------------------------------
//
//  Rev-up complete.
//
//  Please check log (above) to verify that the correct number of
//  DQ and DQS bits were found in your UCF file.
//
//  $mpmc_pcore compatable UCF file located at $output_file.
//  Please rename this file to match the name of your old UCF file.
//  If some of the new constraints cause timing failures, please
//  read text at the end of the new UCF file.  Some of the
//  constraints are intended for very high fMAX and can be relaxed.
//
//  There are new MHS parameters in $mhs_output_file.  Please copy
//  these parameters into your $mpmc_pcore instantiation inside
//  of your MHS file. Connect MPMC_Clk0_DIV2 to an appropriate clock.
//
//  For designs with multiple MPMCs, this script should be executed 
//  multiple times.  The new constraints from each run correspond to
//  a single MPMC instance.  The new constraints in this case must be
//  made more specific by modifying the '*/' wildcard in constraint
//  names to signify the particular MPMC instance the script was 
//  run for. For example, replace '"*/' with '"*MPMC_INST_0/'.
//
//------------------------------------------------------------------
END_OF_SUMMARY

cleanup_exit();

sub cleanup_exit {
    my $exit_status = shift;
    
    #Remove temporary files
    unlink("$output_file_tmp1", "$output_file_tmp2.ucf", "${output_file_tmp2}_hdl_params.txt", "$output_file_tmp3", "$mhs_output_file_tmp", "$part_number.pkg", "partlist.xct", "partlist.xml");

    #Exit with the passed status
    exit($exit_status);
}
