
library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;
library unisim;
use unisim.vcomponents.all;

entity spi_core is
  port (
    SYS_Rst             : in  std_logic;
    SYS_CLK             : in  std_logic;
    SPI_SS              : in  std_logic_vector(0 to 1);
    SPI_MISO            : in  std_logic_vector(0 to 1);
    SPI_MOSI            : out std_logic_vector(0 to 1);
    SPI_CLK             : out std_logic_vector(0 to 1);
    tx_fifo_raed_en     : out std_logic;
    tx_fifo_valid       : in  std_logic;
    tx_fifo_read_data   : in  std_logic_vector(0 to 7);
    rx_fifo_write_en    : out std_logic;
    rx_fifo_write_data  : out std_logic_vector(0 to 7);
    loop_mode           : in  std_logic
  );

end spi_core;


architecture imp of spi_core is

signal tx_fifo_raed_en_w   : std_logic;
signal tx_fifo_raed_en_reg : std_logic;
signal clk_count           : std_logic_vector(11 downto 0);
signal spi_clk_w           : std_logic;
signal clk_sta             : std_logic;
signal spi_clk_reg         : std_logic;
signal MOSI_shiuft         : std_logic_vector(0 to 7);
signal SPI_MISO_w          : std_logic;
signal MISO_shiuft         : std_logic_vector(0 to 7);


begin -- architecture IMP


----------------------------------
-- Clock Start Stop
----------------------------------

process (SYS_CLK) begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_Rst = '1' then
      tx_fifo_raed_en_w <= '0';
    else
        if tx_fifo_valid = '1' and clk_sta = '0' and tx_fifo_raed_en_w = '0'  and tx_fifo_raed_en_reg = '0' then
            tx_fifo_raed_en_w <= '1';
        else
            tx_fifo_raed_en_w <= '0';
        end if;
    end if;
  end if;
end process;

tx_fifo_raed_en <= tx_fifo_raed_en_w;


process (SYS_CLK) begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_Rst = '1' then
      tx_fifo_raed_en_reg <= '0';
    else
        tx_fifo_raed_en_reg <= tx_fifo_raed_en_w;
    end if;
  end if;
end process;


process (SYS_CLK) begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_Rst = '1' then
      clk_sta <= '0';
    else
        if tx_fifo_raed_en_reg = '1' then
            clk_sta <= '1';
        elsif SPI_SS(1) = '0' and  clk_count(4) = '1' then
            clk_sta <= '0';
        elsif SPI_SS(0) = '0' and clk_count(11) = '1' then
            clk_sta <= '0';
        end if;
    end if;
  end if;
end process;


----------------------------------
-- Clock Generator
----------------------------------

process (SYS_CLK) begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_Rst = '1' then
      clk_count <= (others => '0');
    else
        if clk_sta = '0' then
            clk_count <= (others => '0');
        else
            clk_count <= clk_count + 1;
        end if;
    end if;
  end if;
end process;

spi_clk_w <= (clk_count(0) and not clk_count(4) and not SPI_SS(1)) 
or (clk_count(7) and not clk_count(11) and not SPI_SS(0));

SPI_CLK(0) <= spi_clk_w;
SPI_CLK(1) <= spi_clk_w;


process (SYS_CLK) begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_Rst = '1' then
      spi_clk_reg <= '0';
    else
        spi_clk_reg <= spi_clk_w;
    end if;
  end if;
end process;


----------------------------------
-- TX Serial Data Send
----------------------------------

process (SYS_CLK) begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_Rst = '1' then
      MOSI_shiuft <= x"00";
    else
        if tx_fifo_raed_en_reg = '1' then
            MOSI_shiuft <= tx_fifo_read_data;
        elsif (not SPI_SS(1) and spi_clk_w )  = '1' 
                   or (not SPI_SS(0) and not spi_clk_w and spi_clk_reg) = '1' then
            MOSI_shiuft(0 to 6) <= MOSI_shiuft(1 to 7) ;
            MOSI_shiuft(7) <= '0' ;
        end if;
    end if;
  end if;
end process;

SPI_MOSI(0) <= MOSI_shiuft(0);
SPI_MOSI(1) <= MOSI_shiuft(0);

----------------------------------
-- RX Serial Data Recive
----------------------------------
process (SYS_CLK) begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_Rst = '1' then
      rx_fifo_write_en <= '0';
    else
        if (not SPI_SS(1) and clk_count(4) and clk_sta) = '1' 
        or (not SPI_SS(0) and clk_count(11) and clk_sta) = '1' then
            rx_fifo_write_en <= '1';
        else
            rx_fifo_write_en <= '0';
        end if;
    end if;
  end if;
end process;

SPI_MISO_w <= MOSI_shiuft(0) when loop_mode = '1' else
              SPI_MISO(1)    when SPI_SS(1) = '0' else SPI_MISO(0);


process (SYS_CLK) begin
  if SYS_CLK'event and SYS_CLK = '1' then
    if SYS_Rst = '1' then
      MISO_shiuft <= x"00";
    else
        if (not SPI_SS(1) and clk_sta and spi_clk_w)  = '1' 
        or (not SPI_SS(0) and spi_clk_w and not spi_clk_reg) = '1' then
            MISO_shiuft(0 to 6) <= MISO_shiuft(1 to 7) ;
            MISO_shiuft(7) <= SPI_MISO_w ;
        end if;
    end if;
  end if;
end process;

rx_fifo_write_data <= MISO_shiuft;

end architecture imp;
